@@ -9,7 +9,7 @@
 #'
 #' @param observeddata The entire observed dataset
 #' @param treedata A list of dataframes on which the trees are based. Not necessary if the data set is included in the tree object.
-#' @param trees A list of trees of class party, classes inheriting from party (e.g., glmtree), or classes that can be coerced to party (i.e., rpart, Weka_tree, XMLnode).
+#' @param trees A list of trees of class party, classes inheriting from party (e.g., glmtree), classes that can be coerced to party (i.e., rpart, Weka_tree, XMLnode), or a randomForest or ranger object.
 #' @param simmatrix A similaritymatrix with the similarities between all trees. Should be square, symmetric and have ones on the diagonal. Default=NULL
 #' @param m Similarity measure that should be used to calculate similarities, in the case that no similarity matrix was provided by the user. Default=NULL.
 #' m=1 is based on counting common predictors;
@@ -47,6 +47,10 @@
 #' @importFrom graphics axis plot
 #' @importFrom parallel detectCores makeCluster clusterExport parSapply stopCluster
 #' @importFrom igraph graph_from_incidence_matrix max_bipartite_match
+#' @importFrom stats complete.cases
+#' @importFrom methods is
+#' @importFrom randomForest getTree
+#' @importFrom ranger treeInfo
 #' @import MASS
 #' @import partykit
 #' @import rpart
@@ -58,6 +62,7 @@
 #' @examples
 #' require(MASS)
 #' require(rpart)
+#' require(ranger)
 #'#Function to draw a bootstrap sample from a dataset
 #'DrawBoots <- function(dataset, i){
 #'set.seed(2394 + i)
@@ -82,14 +87,35 @@
 #'#Clustering the trees in this forest
 #'ClusterForest<- clusterforest(observeddata=Pima.tr,treedata=Boots,trees=Trees,m=1,
 #'fromclus=1, toclus=5, sameobs=FALSE)
+#'
+#'#Example RandomForest
+#'Pima.tr.ranger <- ranger(type ~ ., data = Pima.tr, keep.inbag = TRUE, num.trees=20,
+#'max.depth=3)
+#'
+#'ClusterForest<- clusterforest(observeddata=Pima.tr,trees=Pima.tr.ranger,m=5,
+#'                            fromclus=1, toclus=2, sameobs=FALSE)
 
 
 clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m=NULL, tol=NULL, weight=NULL,fromclus=1, toclus=1, treecov=NULL, sameobs=FALSE, seed=NULL){
   ############################## Check forest input #####################
   ###  Some checks whether correct forest information is provided by user
+  if(typeof(trees) != "list" ) {
+    cat("trees must be a list of party tree objects, a list of trees that can be converted to party objects, or a randomforest or ranger object")
+    return(NULL)
+  }
   
+  if('ranger' %in% class(trees)){
+      trees<-sapply(1:trees$num.trees, function (k) ranger2party(observeddata, trees, k))
+  }else if('randomForest' %in% class(trees)){
+        trees<-sapply(1:trees$ntree, function (k) randomForest2party(observeddata, trees, k))
+    }else if(!'party' %in% class(trees[[1]])){
+     tryCatch(trees<- lapply(1:length(trees), function (i) as.party(trees[[i]])),
+             error=function(e){cat("trees must be a list of party tree objects or objects that can be coerced to party trees")})
+    }
+  
+   
   if(!is.null(trees[[1]]$data)){
-    treedt=lapply(1:length(trees), function(k) trees[[k]]$data)
+    treedt<- lapply(1:length(trees), function(k) trees[[k]]$data)
   }
   
   if(is.null(trees[[1]]$data)){
@@ -99,7 +125,7 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
       return(NULL)
       }
     
-    if(typeof(treedata) != "list" || class(treedata[[1]]) != "data.frame") {
+    if(typeof(treedata) != "list" || !is(treedata[[1]], "data.frame")) {
     cat("please submit correct treedata: a list of data frames on which the trees were grown")
     return(NULL)
   }
@@ -112,16 +138,6 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
   }
   
   
-  if(typeof(trees) != "list" ) {
-    cat("trees must be a list of party tree objects")
-    return(NULL)
-  }
-  
-  
-  if(!'party' %in% class(trees[[1]])){
-    tryCatch(trees<- lapply(1:length(trees), function (i) as.party(trees[[i]])),
-             error=function(e){cat("trees must be a list of party tree objects or objects that can be coerced to party trees")})
-  }
   
   ## Turn user provided forest information into object of class forest.
   forest <- list(partytrees = trees, treedata = treedt, observeddata=observeddata)
@@ -186,7 +202,7 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
     }
 
     if (m == 3){
-      Xclass<- sapply(1:length(X), function(i) class(forest$observeddata[,X[i]] ) == "integer"|class(forest$observeddata[,X[i]] ) == "numeric")
+      Xclass<- sapply(1:length(X), function(i) is(forest$observeddata[,X[i]], "integer" )|is(forest$observeddata[,X[i]],"numeric" ))
 
       if("FALSE" %in% Xclass){
         cat("This measure only works on numerical splitvariables (class integer or numeric)")
@@ -207,7 +223,7 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
 
 
     if (m == 6){
-      Xclass<- sapply(1:length(X), function(i) class(forest$observeddata[,X[i]] ) == "integer"|class(forest$observeddata[,X[i]] ) == "numeric")
+      Xclass<- sapply(1:length(X), function(i) is(forest$observeddata[,X[i]] , "integer")|is(forest$observeddata[,X[i]], "numeric" )  )
 
       if("FALSE" %in% Xclass){
         cat("This measure only works on numerical splitvariables (class integer or numeric)")
@@ -218,7 +234,7 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
     }
 
     if(m==7){
-      Xclass<- sapply(1:length(X), function(i) class(forest$observeddata[,X[i]] ) == "integer"|class(forest$observeddata[,X[i]] ) == "numeric")
+      Xclass<- sapply(1:length(X), function(i) is(forest$observeddata[,X[i]], "integer" )|is(forest$observeddata[,X[i]], "numeric" ) )
 
       if("FALSE" %in% Xclass){
         cat("This measure only works on numerical splitvariables (class integer or numeric)")
@@ -236,7 +252,7 @@ clusterforest <- function (observeddata, treedata=NULL, trees, simmatrix=NULL, m
     if (m == 8){
 
 
-      Xclass<- sapply(1:length(X), function(i) class(forest$fulldata[,X[i]] ) == "factor")
+      Xclass<- sapply(1:length(X), function(i) is(forest$fulldata[,X[i]], "factor" ) )
 
       if("FALSE" %in% Xclass){
         cat("This measure only works on binary splitvariables (class factor)")
@@ -473,7 +489,7 @@ M6 <- function (observeddata, treedata, Y, X, trees, tol){
   s0<- lapply(1:n, function (k) splitvsets(pamtrees[[k]]$path0))
 
   for (k in 1:n){
-    if(class(treedata[[1]][,Y[1]]) != "factor" ) {
+    if(is(treedata[[1]][,Y[1]], "factor") ) {
       treedata[[k]][,Y[k]] <- as.factor(treedata[[k]][,Y[k]])
     }else{
       treedata[[k]] = treedata[[k]]
@@ -538,8 +554,8 @@ pamtree<- function(observeddata,treedata,Y,tree){
   if(length(tree) > 2){   ## Check whether there was a split
     paths <- listrulesparty(x=tree)   # lists all the paths from root to leave
     prednode <- predict(tree, newdata = observeddata, type = "node")  #predicts node for every row of full data
-    if(class(treedata[,Y]) != "factor" ) {treedata[,Y] <- as.factor(treedata[,Y])}  #check whether y is a factor, if not-- make it a factor
-    if(class(observeddata[,Y]) != "factor" ) {observeddata[,Y] <- as.factor(observeddata[,Y])}  #check whether y is a factor, if not-- make it a factor
+    if(is(treedata[,Y], "factor" )) {treedata[,Y] <- as.factor(treedata[,Y])}  #check whether y is a factor, if not-- make it a factor
+    if(is(observeddata[,Y], "factor") ) {observeddata[,Y] <- as.factor(observeddata[,Y])}  #check whether y is a factor, if not-- make it a factor
 
     #if(class(tree)[1]== "glmtree"){
       predresp <- predict(tree, newdata= observeddata, type="response")  #predicts response for every row of full data
@@ -590,7 +606,7 @@ pamtree<- function(observeddata,treedata,Y,tree){
     path0<- NULL
 
     tree <- list(numeric(0), numeric(0))
-    if(class(treedata[,Y]) != "factor" ) {treedata[,Y] <- as.factor(treedata[,Y])}
+    if(is(treedata[,Y], "factor") ) {treedata[,Y] <- as.factor(treedata[,Y])}
     y <- treedata[, Y]
 
     #check what class is most prevalent
@@ -610,6 +626,143 @@ pamtree<- function(observeddata,treedata,Y,tree){
   return(value)
 }
 
+#### grow party tree (for turning ranger/randomforest tree into partytree)
+grow.party.tree <- function(party.tree, ranger.tree, data, factor.terms.index, currentNodeNumber) {
+  node <- ranger.tree[ranger.tree$nodeID == currentNodeNumber, ]
+  
+  factor.terms.index.left <- factor.terms.index
+  factor.terms.index.right <- factor.terms.index
+  
+  # Create individual node
+  if (node$terminal == TRUE) {
+    newNode <- list(id = node$nodeID)
+  } else {
+    data.is.factor <- is.factor(data[[node$splitvarName]])
+    data.is.ordered <- is.ordered(data[[node$splitvarName]])
+    if (data.is.factor && !data.is.ordered) {
+      index <- factor.terms.index[[node$splitvarName]]
+      index[as.integer(unlist(strsplit(node$splitval, ',')))] = 2L
+      newNode <- list(id = node$nodeID,
+                      split = partysplit(
+                        varid = as.integer(node$splitvarID + 1),
+                        index = index
+                      ), 
+                      kids = c(as.integer(node$leftChild), as.integer(node$rightChild)))
+      factor.terms.index.left[[node$splitvarName]] <- replace(index, index==2L, NA)
+      factor.terms.index.right[[node$splitvarName]] <- replace(replace(index, index==1L, NA), index==2L, 1L)
+    } else {
+      newNode <- list(id = node$nodeID, split = partysplit(varid = as.integer(node$splitvarID + 1), breaks = as.numeric(node$splitval)), kids = c(as.integer(node$leftChild), as.integer(node$rightChild)))
+    }
+  }
+  
+  
+  # Traverse tree recursively
+  if (node$terminal == FALSE) {
+    leftChildren <- grow.party.tree(party.tree, ranger.tree, data, factor.terms.index.left, node$leftChild)
+    rightChildren <- grow.party.tree(party.tree, ranger.tree, data, factor.terms.index.right, node$rightChild)
+    
+    party.tree <- c(party.tree, leftChildren, rightChildren)
+  }
+  
+  # Add newly created node to list of nodes
+  party.tree <- c(party.tree, list(newNode))
+  
+  party.tree
+}
+
+
+generic2party <- function(data, generic.tree, inbag, formula, weights) {
+  response <- all.vars(formula)[1]
+  terms <- terms(formula, data = data)
+  factor.terms <- all.vars(terms)[-1]
+  
+  factor.terms.index <- list()
+  for(factor.term in factor.terms) {
+    factor.terms.index[[factor.term]] <- rep(1L, length(levels(data[[factor.term]])))
+  }
+  
+  data <- data[complete.cases(data), c(all.vars(terms)[-1], response)]
+  
+  data <- as.data.frame(lapply(data, rep, inbag))
+  
+  if (is.null(weights)) {
+    weights <- rep(1L, nrow(data)) 
+  }
+  
+  nodelist = list()
+  nodelist <- grow.party.tree(nodelist, generic.tree, data, factor.terms.index, 0)
+  
+  nodes <- as.partynode(nodelist)
+  fitted <- fitted_node(nodes, data = data)
+  
+  tree <- party(nodes,
+                data = data, 
+                fitted = data.frame("(fitted)" = fitted,
+                                    "(response)" = data[[response]],
+                                    "(weights)" = weights,
+                                    check.names = FALSE),
+                terms = terms(formula, data = data)
+  )
+  as.constparty(tree)
+}
+
+ranger2party <- function(data, ranger.forest, treeNumber, weights = NULL) {
+  if (!exists("inbag.counts", where=ranger.forest)) {
+    stop("Run ranger with the keep.inbag=T parameter")
+  }
+  
+  ranger.tree <- treeInfo(ranger.forest, tree = treeNumber)
+  formula <- formula(ranger.forest$call[[2]])
+  inbag <- ranger.forest$inbag.counts[[treeNumber]]
+  
+  generic2party(data, ranger.tree, inbag, formula, weights)
+}
+
+randomForest2party <- function(data, randomForest.forest, treeNumber, weights = NULL) {
+  if (!exists("inbag", where=randomForest.forest)) {
+    stop("Run randomForest with the keep.inbag=T parameter")
+  }
+  
+  randomForest.tree.without.labels <- data.frame(getTree(randomForest.forest, k = treeNumber, labelVar = F))
+  randomForest.tree.with.labels <- data.frame(getTree(randomForest.forest, k = treeNumber, labelVar = T))
+  
+  # Convert randomForest format to Ranger format  
+  colnames(randomForest.tree.with.labels) <- c("leftChild", "rightChild", "splitvarName", "splitval", "terminal", "prediction")
+  colnames(randomForest.tree.without.labels) <- c("leftChild", "rightChild", "splitvarID", "splitval", "terminal", "prediction")
+  nodeID <- 0:(nrow(randomForest.tree.with.labels) -1)
+  leftChild <- randomForest.tree.with.labels$leftChild - 1L
+  rightChild <- randomForest.tree.with.labels$rightChild - 1L
+  splitvarID <- randomForest.tree.without.labels$splitvarID - 1L
+  splitvarName <- as.character(randomForest.tree.with.labels$splitvarName)
+  splitval <- randomForest.tree.with.labels$splitval
+  terminal <- ifelse(randomForest.tree.without.labels$terminal == -1, TRUE, FALSE)
+  is.na(rightChild) <- is.na(splitvarID) <- is.na(leftChild) <- is.na(splitval) <- terminal
+  prediction <- randomForest.tree.with.labels$prediction
+  generic.tree <- data.frame(nodeID, leftChild, rightChild, splitvarID, splitvarName, splitval, terminal, prediction)
+  
+  idx.unordered <- apply(array(splitvarName), 1, function(x) { !("ordered" %in% class(data[[x]]) || "numeric" %in% class(data[[x]]))})
+  idx.unordered[terminal] <- FALSE
+  
+  if (any(idx.unordered)) {
+    if (any(generic.tree$splitval[idx.unordered] > (2^31 - 1))) {
+      warning("Unordered splitting levels can only be shown for up to 31 levels.")
+      generic.tree$splitval[idx.unordered] <- NA
+    } else {
+      generic.tree$splitval[idx.unordered] <- sapply(generic.tree$splitval[idx.unordered], function(x) {
+        paste(which(as.logical(intToBits(2^31-1-x))), collapse = ",")
+      })
+    }
+  }  
+  
+  formula <- formula(randomForest.forest$call[[2]])
+  
+  inbag <- randomForest.forest$inbag[, treeNumber]
+  
+  generic2party(data, generic.tree, inbag, formula, weights)
+}
+
+
+
 
 
 ### partykit:::.list.rules.party
